/*
 * Calculs.cpp : Application schemas explicite sur les objets.
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file Calculs.cpp
    Programme calculant pour chaque particule i 
    son etat au pas de temps suivant (methode d 'Euler explicite) :    
    principales fonctions de calculs.  
    \brief Fonctions de calculs de la methode explicite
*/ 

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <iostream>

#include "Donnees.h"
#include "ObjetSimule.h"

extern int trackballBeginX,trackballBeginY;
/**
 * Calcul des forces appliquees sur les particules.
 */
void ObjetSimule::CalculForce()
{
 
  
  Coord fe,fv;
  Particule* p;
  Particule* p2;
  Ressort* r;
  //  double distance;
  //  Coord direction;
  double fMax=0;
  for (unsigned int i = 0; i < _Nb_Particules; i++){

    p = Maillage->GetParticule(i);
    fe = Coord(0.0, 0.0, 0.0);
    fv = Coord(0.0, 0.0, 0.0);
    for(int j=0; j<p->GetRessortList().size(); j++){

      r = p->GetRessortList()[j];
	
      if(r->GetParticuleA() == p)
	p2 = r->GetParticuleB();
      else
	p2 = r->GetParticuleA();

      Coord v(p2->_Pos.getX() - p->_Pos.getX(), p2->_Pos.getY() - p->_Pos.getY(), p2->_Pos.getZ() - p->_Pos.getZ());
      Coord w(V[p->GetId()].getX() - V[i].getX(), V[p->GetId()].getY() - V[i].getY(), V[p->GetId()].getZ() - V[i].getZ());
	
      fe = Normalized(v) * r->GetRaideur() * ( Norm(v) - r->GetLrepos());
      fv = Normalized(v) * (Norm(w) * r->GetAmortissement());
      Force[i] += fe + fv + Colision;
      fMax = Norm(Force[i])>fMax ? Norm(Force[i]) : fMax;
      //      std::cout<<Norm(Force[i])<<std::endl;
      if(_dechirure && Norm(Force[i])>_dechirure /* && i> _Nb_Particules/10*/){
	//      	 std::cout<<"Break"<<i<<std::endl;
	 
      	int pos;
      	for(pos=0;pos<p2->_RessortList.size();pos++)
      	  if(p2->_RessortList[pos] == r)
      	    break;
      	p2->_RessortList.erase(p2->_RessortList.begin()+pos);

      	for(pos=0;pos<p->_RessortList.size();pos++)
      	  if(p->_RessortList[pos] == r)
      	    break;
      	p->_RessortList.erase(p->_RessortList.begin()+pos);
	r->_ParticuleA = NULL;
	r->_ParticuleB = NULL;

      }
      
    }
  }
  //  std::cout<<fMax<<std::endl;
}//void


/**
 * Calcul de l acceleration des particules
 * avec ajout de la gravite aux forces des particules
 * et ajout de la force due au vent sur une des particules du maillage
 * et reinitialisation des forces.
 */
void ObjetSimule::CalculAccel(Coord g)
{
  //appliquer la même force à plusieurs points
  int minWind = -50;
  int maxWind = 20;
  int vX = 0;//vent?minWind+rand()%(maxWind-minWind):0;
  int vY = vent?minWind+rand()%(maxWind-minWind):0;
  int vZ = vent?minWind+rand()%(maxWind-minWind):0;
  Coord vent(vX,vY,vZ);
  // int imin = 0;
  // int imax = _Nb_Particules -1;

  // for(int i = 0; i<100; i++){
  //   int indice = imin+rand()%(imax-imin);
  //   Force[indice] += Coord(vX,vY,vZ);
  // }
  for(unsigned int i=0; i<_Nb_Particules; i++){
    A[i] = M[i] ? (Force[i]+vent) * (1./M[i]) + g : Coord(0,0,0);
    Force[i] = Coord(0,0,0);
  }


}//void


/**
 * Calcul de la position et de la vitesse des particules 
 * au temps 0 (methode d integration Leap Frog).       
 */
void ObjetSimule::CalculPositionVitesseTps0(float visco)
{
  for(int i=0; i<_Nb_Particules; i++)
    V[i] += A[i]*(_delta_t/2)*visco;
}


/**
 * Calcul de la position et de la vitesse des particules.     
 */
void ObjetSimule::CalculPositionVitesse(float visco, std::vector<BBox*> vbbox, std::vector<tree*> vt/*std::vector<BSH*> vbs*/)
{

  /*
    double sy = trackballBeginY;
    double sx = trackballBeginX;



    GLdouble modelMatrix[16];
    GLdouble projMatrix[16];
    GLint viewport[4];

    glGetIntegerv(GL_VIEWPORT, viewport);
    glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);

    GLdouble x, y, z;
    gluUnProject(sx, viewport[1] + viewport[3] - sy, 0, modelMatrix, projMatrix, viewport, &x, &y, &z);

    GLfloat nearv[3], farv[3];  // already computed as above

    if(nearv[2] == farv[2])     // this means we have no solutions
    return;

    GLfloat t = (nearv[2] - z) / (nearv[2] - farv[2]);

    // so here are the desired (x, y) coordinates
    GLfloat xW = nearv[0] + (farv[0] - nearv[0]) * t,
    yW = nearv[1] + (farv[1] - nearv[1]) * t;



    P[0] = Coord(xW, yW, 0);
    P[0].Print();*/
  //  if()

  //  

  int col = 0;
  for (unsigned int i = 0; i < _Nb_Particules; i++){
    int ok=1;
    Coord old = P[i];
    V[i] += A[i]*_delta_t*visco;
    //    P[i] += V[i]*_delta_t;
    //si la particule n'est pas rejetté par une boite englobante...
    for(unsigned int j=0;j<vbbox.size();j++){
      if(vbbox[j]->inside(P[i])){
       	ok=0;
	//	P[i] = old;
      }
    }
    for(unsigned int j=0;j<vt.size();j++){
      if(vt[j]->inside(P[i])){
       	ok=0;
	//	P[i] = old;
      }
    }
   
    
    
    if(!ok){
      Colision +=V[i]*_delta_t*-_rebond;
      col++; 
      P[i] = old;
    }else
      P[i] += V[i]*_delta_t;

    //mettre à jour le maillage et l'englobant
   
    Maillage->_ParticuleList[i]->SetPosition(P[i]);
    
  }
   if(!col)
     Colision = Coord(0,0,0);

  // int col = 0;

  // for (unsigned int i = 0; i < _Nb_Particules; i++){
  //   int ok=1;
  //   Coord old = P[i];
  //   //    P[i] += V[i]*_delta_t;
  //   V[i] += A[i]*_delta_t*visco;
  //   //si la particule n'est pas rejetté par une boite englobante...
  //   // for(unsigned int j=0;j<vt.size();j++){
  //   //   if(vt[j]->inside(P[i])){
  //   //    	ok=0;
  //   // 	col++; 
  //   // 	//	P[i] = old;
  //   //   }
  //   // }
  //   for(unsigned int j=0;j<vbbox.size();j++){
  //     if(vbbox[j]->inside(P[i])){
  //      	ok=0;
  // 	col++; 
  //     }
  //   }
  //   if(!ok)
  //     P[i] -= V[i]*_delta_t;
  //   //    Colision +=V[i]*_delta_t*-_rebond; 
  // }
  // for (unsigned int i = 0; i < _Nb_Particules; i++){  
  //   P[i] += V[i]*_delta_t + Colision;  
  //   Maillage->_ParticuleList[i]->SetPosition(P[i]);    
  // }
    
  // std::cout<<col<<std::endl;
  // if(!col)
  //   Colision = Coord(0,0,0);

}//void
